#!/usr/bin/env python3
"""
Firewall Analyzer
Advanced tool for analyzing firewall rules and configurations.
Parameters:
    - mode: Analysis mode (rules, config, audit)
    - target: Target firewall or config file
Requires: admin/root privileges
"""
import subprocess
import platform
import json
import logging
from datetime import datetime
import re
import argparse
import sys
from pathlib import Path

class FirewallAnalyzer:
    def __init__(self, mode='rules', target=None):
        self.mode = mode
        self.target = target
        self.is_windows = platform.system().lower() == 'windows'
        self.rules = []
        self.setup_logging()
        
    def setup_logging(self):
        """Set up logging configuration"""
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s',
            handlers=[
                logging.FileHandler('firewall_analysis.log'),
                logging.StreamHandler()
            ]
        )
        
    def get_windows_rules(self):
        """Get Windows Firewall rules"""
        try:
            cmd = ['netsh', 'advfirewall', 'show', 'currentprofile']
            result = subprocess.run(cmd, capture_output=True, text=True)
            if result.returncode == 0:
                self.rules.append(('Profile', result.stdout))
                
            cmd = ['netsh', 'advfirewall', 'firewall', 'show', 'rule', 'name=all']
            result = subprocess.run(cmd, capture_output=True, text=True)
            if result.returncode == 0:
                self.rules.append(('Rules', result.stdout))
                
        except subprocess.SubprocessError as e:
            logging.error(f"Error getting Windows Firewall rules: {e}")
            
    def get_linux_rules(self):
        """Get Linux iptables rules"""
        try:
            cmd = ['iptables', '-L', '-n', '-v']
            result = subprocess.run(cmd, capture_output=True, text=True)
            if result.returncode == 0:
                self.rules.append(('IPv4 Rules', result.stdout))
                
            cmd = ['ip6tables', '-L', '-n', '-v']
            result = subprocess.run(cmd, capture_output=True, text=True)
            if result.returncode == 0:
                self.rules.append(('IPv6 Rules', result.stdout))
                
        except subprocess.SubprocessError as e:
            logging.error(f"Error getting Linux firewall rules: {e}")
            
    def analyze_rules(self):
        """Analyze firewall rules"""
        if self.is_windows:
            self.get_windows_rules()
        else:
            self.get_linux_rules()
            
        # Analyze collected rules
        findings = []
        for rule_type, rules in self.rules:
            findings.extend(self.analyze_rule_set(rule_type, rules))
        return findings
    
    def analyze_rule_set(self, rule_type, rules):
        """Analyze a set of firewall rules"""
        findings = []
        
        # Check for common security issues
        if 'any' in rules.lower():
            findings.append({
                'type': 'warning',
                'rule_type': rule_type,
                'message': 'Found rules with "any" source/destination'
            })
            
        if 'allow' in rules.lower() and 'inbound' in rules.lower():
            findings.append({
                'type': 'info',
                'rule_type': rule_type,
                'message': 'Found inbound allow rules - verify if necessary'
            })
            
        # Check for disabled firewall
        if 'disabled' in rules.lower() or 'off' in rules.lower():
            findings.append({
                'type': 'critical',
                'rule_type': rule_type,
                'message': 'Firewall appears to be disabled'
            })
            
        return findings
    
    def analyze_config(self):
        """Analyze firewall configuration file"""
        if not self.target or not Path(self.target).exists():
            logging.error("No valid configuration file provided")
            return []
            
        try:
            with open(self.target, 'r') as f:
                config = f.read()
                return self.analyze_rule_set('Config File', config)
        except Exception as e:
            logging.error(f"Error reading configuration file: {e}")
            return []
            
    def audit_firewall(self):
        """Perform security audit of firewall"""
        findings = []
        
        # Check basic security settings
        findings.extend(self.analyze_rules())
        
        # Additional security checks
        if self.is_windows:
            # Check Windows Firewall service
            cmd = ['sc', 'query', 'MpsSvc']
            try:
                result = subprocess.run(cmd, capture_output=True, text=True)
                if 'RUNNING' not in result.stdout:
                    findings.append({
                        'type': 'critical',
                        'rule_type': 'Service',
                        'message': 'Windows Firewall service is not running'
                    })
            except subprocess.SubprocessError as e:
                logging.error(f"Error checking firewall service: {e}")
        
        return findings
    
    def analyze(self):
        """Main analysis method"""
        logging.info(f"Starting firewall analysis in {self.mode} mode")
        
        if self.mode == 'rules':
            findings = self.analyze_rules()
        elif self.mode == 'config':
            findings = self.analyze_config()
        elif self.mode == 'audit':
            findings = self.audit_firewall()
        else:
            logging.error(f"Invalid mode: {self.mode}")
            return
            
        # Save findings
        self.save_findings(findings)
        
        return findings
    
    def save_findings(self, findings):
        """Save analysis findings to file"""
        output_file = f'firewall_analysis_{datetime.now().strftime("%Y%m%d_%H%M%S")}.json'
        try:
            with open(output_file, 'w') as f:
                json.dump(findings, f, indent=4)
            logging.info(f"Analysis results saved to {output_file}")
        except Exception as e:
            logging.error(f"Error saving findings: {e}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Firewall Analyzer")
    parser.add_argument("--mode", choices=['rules', 'config', 'audit'],
                      default='rules', help="Analysis mode")
    parser.add_argument("--target", help="Target firewall or config file")
    args = parser.parse_args()
    
    analyzer = FirewallAnalyzer(mode=args.mode, target=args.target)
    findings = analyzer.analyze()
    
    # Print summary to console
    print("\nAnalysis Summary:")
    for finding in findings:
        print(f"[{finding['type'].upper()}] {finding['rule_type']}: {finding['message']}")
